@@ -7,8 +7,9 @@
     using K/V semantics like _get and _put.
 """
 from __future__ import absolute_import, unicode_literals
+from future.utils import raise_with_traceback
 
-import datetime
+from datetime import datetime, timedelta
 import sys
 import time
 import warnings
@@ -26,7 +27,8 @@ import celery.exceptions
 from celery import current_app, group, maybe_signature, states
 from celery._state import get_current_task
 from celery.exceptions import (ChordError, ImproperlyConfigured,
-                               NotRegistered, TaskRevokedError, TimeoutError)
+                               NotRegistered, TaskRevokedError, TimeoutError,
+                               BackendGetMetaError, BackendStoreError)
 from celery.five import PY3, items
 from celery.result import (GroupResult, ResultBase, ResultSet,
                            allow_join_result, result_from_tuple)
@@ -37,6 +39,7 @@ from celery.utils.serialization import (create_exception_cls,
                                         ensure_serializable,
                                         get_pickleable_exception,
                                         get_pickled_exception)
+from celery.utils.time import get_exponential_backoff_interval
 
 __all__ = ('BaseBackend', 'KeyValueStoreBackend', 'DisabledBackend')
 
@@ -126,6 +129,11 @@ class Backend(object):
         self.accept = conf.accept_content if self.accept is None else self.accept  # noqa: E501
         self.accept = prepare_accept_content(self.accept)
 
+        self.always_retry = conf.get('result_backend_always_retry', False)
+        self.max_sleep_between_retries_ms = conf.get('result_backend_max_sleep_between_retries_ms', 10000)
+        self.base_sleep_between_retries_ms = conf.get('result_backend_base_sleep_between_retries_ms', 10)
+        self.max_retries = conf.get('result_backend_max_retries', float("inf"))
+
         self._pending_results = pending_results_t({}, WeakValueDictionary())
         self._pending_messages = BufferMap(MESSAGE_BUFFER_MAX)
         self.url = url
@@ -347,7 +355,7 @@ class Backend(object):
     def prepare_expires(self, value, type=None):
         if value is None:
             value = self.app.conf.result_expires
-        if isinstance(value, datetime.timedelta):
+        if isinstance(value, timedelta):
             value = value.total_seconds()
         if value is not None and type:
             return type(value)
@@ -371,7 +379,7 @@ class Backend(object):
                          state, traceback, request, format_date=True,
                          encode=False):
         if state in self.READY_STATES:
-            date_done = datetime.datetime.utcnow()
+            date_done = datetime.utcnow()
             if format_date:
                 date_done = date_done.isoformat()
         else:
@@ -415,13 +423,40 @@ class Backend(object):
 
         return meta
 
+    def _sleep(self, amount):
+        time.sleep(amount)
+
     def store_result(self, task_id, result, state,
                      traceback=None, request=None, **kwargs):
-        """Update task state and result."""
+        """Update task state and result.
+
+        if always_retry_backend_operation is activated, in the event of a recoverable exception,
+        then retry operation with an exponential backoff until a limit has been reached.
+        """
         result = self.encode_result(result, state)
-        self._store_result(task_id, result, state, traceback,
-                           request=request, **kwargs)
-        return result
+
+        retries = 0
+
+        while True:
+            try:
+                self._store_result(task_id, result, state, traceback,
+                                   request=request, **kwargs)
+                return result
+            except Exception as exc:
+                if self.always_retry and self.exception_safe_to_retry(exc):
+                    if retries < self.max_retries:
+                        retries += 1
+
+                        # get_exponential_backoff_interval computes integers
+                        # and time.sleep accept floats for sub second sleep
+                        sleep_amount = get_exponential_backoff_interval(
+                            self.base_sleep_between_retries_ms, retries,
+                            self.max_sleep_between_retries_ms, True) / 1000
+                        self._sleep(sleep_amount)
+                    else:
+                        raise_with_traceback(BackendStoreError("failed to store result on the backend", task_id=task_id, state=state))
+                else:
+                    raise
 
     def forget(self, task_id):
         self._cache.pop(task_id, None)
@@ -458,15 +493,49 @@ class Backend(object):
                 RuntimeWarning
             )
 
+    def exception_safe_to_retry(self, exc):
+        """Check if an exception is safe to retry.
+
+        Backends have to overload this method with correct predicates dealing with their exceptions.
+
+        By default no exception is safe to retry, it's up to backend implementation
+        to define which exceptions are safe.
+        """
+        return False
+
     def get_task_meta(self, task_id, cache=True):
+        """Get task meta from backend.
+
+        if always_retry_backend_operation is activated, in the event of a recoverable exception,
+        then retry operation with an exponential backoff until a limit has been reached.
+        """
         self._ensure_not_eager()
         if cache:
             try:
                 return self._cache[task_id]
             except KeyError:
                 pass
+        retries = 0
+        while True:
+            try:
+                meta = self._get_task_meta_for(task_id)
+                break
+            except Exception as exc:
+                if self.always_retry and self.exception_safe_to_retry(exc):
+                    if retries < self.max_retries:
+                        retries += 1
+
+                        # get_exponential_backoff_interval computes integers
+                        # and time.sleep accept floats for sub second sleep
+                        sleep_amount = get_exponential_backoff_interval(
+                            self.base_sleep_between_retries_ms, retries,
+                            self.max_sleep_between_retries_ms, True) / 1000
+                        self._sleep(sleep_amount)
+                    else:
+                        raise_with_traceback(BackendGetMetaError("failed to get meta", task_id=task_id))
+                else:
+                    raise
 
-        meta = self._get_task_meta_for(task_id)
         if cache and meta.get('status') == states.SUCCESS:
             self._cache[task_id] = meta
         return meta
@@ -666,6 +735,9 @@ class BaseKeyValueStoreBackend(Backend):
     def mget(self, keys):
         raise NotImplementedError('Does not support get_many')
 
+    def _set_with_state(self, key, value, state):
+        return self.set(key, value)
+
     def set(self, key, value):
         raise NotImplementedError('Must implement the set method.')
 
@@ -786,12 +858,12 @@ class BaseKeyValueStoreBackend(Backend):
         if current_meta['status'] == states.SUCCESS:
             return result
 
-        self.set(self.get_key_for_task(task_id), self.encode(meta))
+        self._set_with_state(self.get_key_for_task(task_id), self.encode(meta), state)
         return result
 
     def _save_group(self, group_id, result):
-        self.set(self.get_key_for_group(group_id),
-                 self.encode({'result': result.as_tuple()}))
+        self._set_with_state(self.get_key_for_group(group_id),
+                             self.encode({'result': result.as_tuple()}), states.SUCCESS)
         return result
 
     def _delete_group(self, group_id):
