@@ -4,6 +4,8 @@ from __future__ import absolute_import, unicode_literals
 
 import os
 
+from celery import signals
+
 from .base import BasePool, apply_target
 
 __all__ = ('TaskPool',)
@@ -18,6 +20,7 @@ class TaskPool(BasePool):
         super(TaskPool, self).__init__(*args, **kwargs)
         self.on_apply = apply_target
         self.limit = 1
+        signals.worker_process_init.send(sender=None)
 
     def _get_info(self):
         return {
