@@ -7,7 +7,9 @@ If you're looking for the decorator implementation then that's in
 ``celery.app.base.Celery.task``.
 """
 from __future__ import absolute_import, unicode_literals
-from celery._state import current_app, current_task as current
+
+from celery._state import current_app
+from celery._state import current_task as current
 from celery.local import LazyModule, Proxy, recreate_module
 
 __all__ = (
