@@ -1,12 +1,10 @@
 from __future__ import absolute_import, unicode_literals
 
-from django.conf.urls.defaults import (  # noqa
-    url, patterns, include, handler404, handler500,
-)
-
-from djcelery import views as celery_views
+from django.conf.urls.defaults import (handler404, handler500,  # noqa
+                                       include, patterns, url)
 
 from celery_http_gateway.tasks import hello_world
+from djcelery import views as celery_views
 
 # Uncomment the next two lines to enable the admin:
 # from django.contrib import admin
