@@ -1,4 +1,7 @@
 from __future__ import absolute_import, unicode_literals
+
 from django.db import models  # noqa
 
-# Create your models here.
+
+class Widget(models.Model):
+    name = models.CharField(max_length=140)
