@@ -27,7 +27,8 @@ name using the fully qualified form::
     $ celery -A myapp:app worker -l info
 
 """
-from __future__ import absolute_import, unicode_literals, print_function
+from __future__ import absolute_import, print_function, unicode_literals
+
 from celery import Celery
 
 app = Celery(
