@@ -2,6 +2,7 @@
 from __future__ import absolute_import, unicode_literals
 
 import os
+import pickle
 import tempfile
 
 import pytest
@@ -89,3 +90,8 @@ class test_FilesystemBackend:
         tb.mark_as_done(tid, 42)
         tb.forget(tid)
         assert len(os.listdir(self.directory)) == 0
+
+    @pytest.mark.usefixtures('depends_on_current_app')
+    def test_pickleable(self):
+        tb = FilesystemBackend(app=self.app, url=self.url, serializer='pickle')
+        assert pickle.loads(pickle.dumps(tb))
