@@ -6,7 +6,10 @@ from pickle import dumps, loads
 import pytest
 from case import ANY, MagicMock, Mock, mock, patch, sentinel, skip
 from kombu.exceptions import EncodeError
-from pymongo.errors import ConfigurationError
+try:
+    from pymongo.errors import ConfigurationError
+except ImportError:
+    ConfigurationError = None
 
 from celery import states, uuid
 from celery.backends.mongodb import InvalidDocument, MongoBackend
